@@ -140,10 +140,6 @@ typedef enum
   GSD_INPUT_SOURCES_SWITCHER_ALT_SHIFT,
   GSD_INPUT_SOURCES_SWITCHER_CTRL_SHIFT,
   GSD_INPUT_SOURCES_SWITCHER_ALT_CTRL,
-  GSD_INPUT_SOURCES_SWITCHER_CAPS,
-  GSD_INPUT_SOURCES_SWITCHER_SHIFT_CAPS,
-  GSD_INPUT_SOURCES_SWITCHER_ALT_CAPS,
-  GSD_INPUT_SOURCES_SWITCHER_CTRL_CAPS,
 } GsdInputSourcesSwitcher;
 
 #endif /* __gsd_enums_h__ */
