@@ -44,6 +44,7 @@ typedef struct {
         const char *name;       /* property name */
         gint nitems;            /* number of items in data */
         gint format;            /* CARD8 or CARD32 sized-items */
+        gint type;              /* Atom representing data type */
         union {
                 const gchar *c; /* 8 bit data */
                 const gint *i;  /* 32 bit data */
@@ -60,9 +61,13 @@ gboolean  device_is_touchpad       (XDevice                *xdevice);
 
 gboolean  device_info_is_touchpad    (XDeviceInfo         *device_info);
 gboolean  device_info_is_touchscreen (XDeviceInfo         *device_info);
+gboolean  device_info_is_mouse       (XDeviceInfo         *device_info);
+gboolean  device_info_is_trackball   (XDeviceInfo         *device_info);
 
 gboolean  touchpad_is_present     (void);
 gboolean  touchscreen_is_present  (void);
+gboolean  mouse_is_present        (void);
+gboolean  trackball_is_present    (void);
 
 gboolean  device_set_property     (XDevice                *xdevice,
                                    const char             *device_name,
