@@ -54,9 +54,9 @@ print_disabled_devices (void)
 int main (int argc, char **argv)
 {
 	gboolean supports_xinput;
-	gboolean has_touchpad, has_touchscreen;
+	gboolean has_touchpad, has_touchscreen, has_trackball;
         XDeviceInfo *device_info;
-        gint n_devices;
+        gint n_devices, opcode;
         guint i;
 
 	gtk_init (&argc, &argv);
@@ -68,9 +68,9 @@ int main (int argc, char **argv)
 		g_print ("Supports XInput:\t\t\tno\n");
 		return 0;
 	}
-	supports_xinput = supports_xinput2_devices ();
+	supports_xinput = supports_xinput2_devices (&opcode);
 	if (supports_xinput) {
-		g_print ("Supports XInput2:\t\t\tyes\n");
+		g_print ("Supports XInput2:\t\t\tyes (opcode: %d)\n", opcode);
 	} else {
 		g_print ("Supports XInput2:\t\t\tno\n");
 		return 0;
@@ -82,6 +82,9 @@ int main (int argc, char **argv)
 	has_touchscreen = touchscreen_is_present ();
 	g_print ("Has touchscreen:\t\t\t%s\n", has_touchscreen ? "yes" : "no");
 
+	has_trackball = trackball_is_present ();
+	g_print ("Has trackball:\t\t\t\t%s\n", has_trackball ? "yes" : "no");
+
         device_info = XListInputDevices (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), &n_devices);
         if (device_info == NULL) {
 		g_warning ("Has no input devices");
@@ -97,6 +100,10 @@ int main (int argc, char **argv)
 			g_print ("Device %d is touchscreen:\t\t%s\n", (int) device_info[i].id, "yes");
 			continue;
 		}
+		if (device_info_is_trackball (&device_info[i])) {
+			g_print ("Device %d is trackball:\t\t\t%s\n", (int) device_info[i].id, "yes");
+			continue;
+		}
 
                 gdk_error_trap_push ();
                 device = XOpenDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device_info[i].id);
@@ -105,8 +112,15 @@ int main (int argc, char **argv)
 
                 if (device_is_touchpad (device))
 			g_print ("Device %d is touchpad:\t\t%s\n", (int) device_info[i].id, "yes");
-		else
-			g_print ("Device %d is touchpad/touchscreen:\t%s\n", (int) device_info[i].id, "no");
+		else {
+			int tool_id;
+
+			tool_id = xdevice_get_last_tool_id (device_info[i].id);
+			if (tool_id >= 0x0)
+				g_print ("Device %d is touchpad/touchscreen:\t%s (tool ID: 0x%x)\n", (int) device_info[i].id, "no", tool_id);
+			else
+				g_print ("Device %d is touchpad/touchscreen:\t%s\n", (int) device_info[i].id, "no");
+		}
 
                 XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device);
         }
