@@ -49,6 +49,7 @@
 #include <ibus.h>
 #endif
 
+#include "gnome-settings-session.h"
 #include "gnome-settings-profile.h"
 #include "gsd-keyboard-manager.h"
 #include "gsd-input-helper.h"
@@ -232,8 +233,7 @@ fetch_ibus_engines_result (GObject            *object,
                                                    result,
                                                    &error);
         if (!list && error) {
-                if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
-                        g_warning ("Couldn't finish IBus request: %s", error->message);
+                g_warning ("Couldn't finish IBus request: %s", error->message);
                 g_error_free (error);
 
                 clear_ibus (manager);
@@ -323,83 +323,6 @@ maybe_start_ibus (GsdKeyboardManager *manager,
                                               NULL));
 }
 
-static void
-got_session_name (GObject            *object,
-                  GAsyncResult       *res,
-                  GsdKeyboardManager *manager)
-{
-        GVariant *result, *variant;
-        GDBusConnection *connection = G_DBUS_CONNECTION (object);
-        GsdKeyboardManagerPrivate *priv = manager->priv;
-        const gchar *session_name = NULL;
-        GError *error = NULL;
-
-        /* IBus shouldn't have been touched yet */
-        g_return_if_fail (priv->ibus == NULL);
-
-        g_clear_object (&priv->ibus_cancellable);
-
-        result = g_dbus_connection_call_finish (connection, res, &error);
-        if (!result) {
-                g_warning ("Couldn't get session name: %s", error->message);
-                g_error_free (error);
-                goto out;
-        }
-
-        g_variant_get (result, "(v)", &variant);
-        g_variant_unref (result);
-
-        g_variant_get (variant, "&s", &session_name);
-
-        if (g_strcmp0 (session_name, "gnome") == 0)
-                manager->priv->session_is_fallback = FALSE;
-
-        g_variant_unref (variant);
- out:
-        apply_input_sources_settings (manager->priv->input_sources_settings, NULL, 0, manager);
-        g_object_unref (connection);
-}
-
-static void
-got_bus (GObject            *object,
-         GAsyncResult       *res,
-         GsdKeyboardManager *manager)
-{
-        GDBusConnection *connection;
-        GsdKeyboardManagerPrivate *priv = manager->priv;
-        GError *error = NULL;
-
-        /* IBus shouldn't have been touched yet */
-        g_return_if_fail (priv->ibus == NULL);
-
-        g_clear_object (&priv->ibus_cancellable);
-
-        connection = g_bus_get_finish (res, &error);
-        if (!connection) {
-                g_warning ("Couldn't get session bus: %s", error->message);
-                g_error_free (error);
-                apply_input_sources_settings (priv->input_sources_settings, NULL, 0, manager);
-                return;
-        }
-
-        priv->ibus_cancellable = g_cancellable_new ();
-
-        g_dbus_connection_call (connection,
-                                "org.gnome.SessionManager",
-                                "/org/gnome/SessionManager",
-                                "org.freedesktop.DBus.Properties",
-                                "Get",
-                                g_variant_new ("(ss)",
-                                               "org.gnome.SessionManager",
-                                               "SessionName"),
-                                NULL,
-                                G_DBUS_CALL_FLAGS_NONE,
-                                -1,
-                                priv->ibus_cancellable,
-                                (GAsyncReadyCallback)got_session_name,
-                                manager);
-}
-
 static void
 set_ibus_engine_finish (GObject            *object,
                         GAsyncResult       *res,
@@ -414,8 +337,7 @@ set_ibus_engine_finish (GObject            *object,
 
         result = ibus_bus_set_global_engine_async_finish (ibus, res, &error);
         if (!result) {
-                if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
-                        g_warning ("Couldn't set IBus engine: %s", error->message);
+                g_warning ("Couldn't set IBus engine: %s", error->message);
                 g_error_free (error);
         }
 }
@@ -612,11 +534,8 @@ convert_ibus (GSettings *settings)
 
         ibus_settings = g_settings_new ("org.freedesktop.ibus.general");
         engines = g_settings_get_strv (ibus_settings, "preload-engines");
-        for (e = engines; *e; ++e) {
-                if (g_str_has_prefix (*e, "xkb:"))
-                        continue;
+        for (e = engines; *e; ++e)
                 g_variant_builder_add (&builder, "(ss)", INPUT_SOURCE_TYPE_IBUS, *e);
-        }
 
         g_settings_set_value (settings, KEY_INPUT_SOURCES, g_variant_builder_end (&builder));
 
@@ -1314,11 +1233,10 @@ settings_changed (GSettings          *settings,
 	    g_strcmp0 (key, KEY_BELL_MODE) == 0) {
 		g_debug ("Bell setting '%s' changed, applying bell settings", key);
 		apply_bell (manager);
-	} else if (g_strcmp0 (key, KEY_REMEMBER_NUMLOCK_STATE) == 0) {
-		g_debug ("Remember Num-Lock state '%s' changed, applying num-lock settings", key);
+	} else if (g_strcmp0 (key, KEY_REMEMBER_NUMLOCK_STATE) == 0 ||
+		 g_strcmp0 (key, KEY_NUMLOCK_STATE) == 0) {
+		g_debug ("Num-Lock state '%s' changed, applying num-lock state settings", key);
 		apply_numlock (manager);
-	} else if (g_strcmp0 (key, KEY_NUMLOCK_STATE) == 0) {
-		g_debug ("Num-Lock state '%s' changed, will apply at next startup", key);
 	} else if (g_strcmp0 (key, KEY_REPEAT) == 0 ||
 		 g_strcmp0 (key, KEY_INTERVAL) == 0 ||
 		 g_strcmp0 (key, KEY_DELAY) == 0) {
@@ -1342,7 +1260,6 @@ device_added_cb (GdkDeviceManager   *device_manager,
         source = gdk_device_get_source (device);
         if (source == GDK_SOURCE_KEYBOARD) {
                 g_debug ("New keyboard plugged in, applying all settings");
-                apply_all_settings (manager);
                 apply_input_sources_settings (manager->priv->input_sources_settings, NULL, 0, manager);
                 run_custom_command (device, COMMAND_DEVICE_ADDED);
         }
@@ -1567,6 +1484,10 @@ maybe_create_input_sources (GsdKeyboardManager *manager)
 static gboolean
 start_keyboard_idle_cb (GsdKeyboardManager *manager)
 {
+        GDBusProxy *proxy;
+        GVariant *prop;
+        const gchar *name;
+
         gnome_settings_profile_start (NULL);
 
         g_debug ("Starting keyboard manager");
@@ -1583,18 +1504,19 @@ start_keyboard_idle_cb (GsdKeyboardManager *manager)
 
         maybe_create_input_sources (manager);
 
-#ifdef HAVE_IBUS
-        /* We don't want to touch IBus until we are sure this isn't a
-           fallback session. */
-        manager->priv->session_is_fallback = TRUE;
-        manager->priv->ibus_cancellable = g_cancellable_new ();
-        g_bus_get (G_BUS_TYPE_SESSION,
-                   manager->priv->ibus_cancellable,
-                   (GAsyncReadyCallback)got_bus,
-                   manager);
-#else
+        proxy = gnome_settings_session_get_session_proxy ();
+        prop = g_dbus_proxy_get_cached_property (proxy, "session-name");
+        if (prop) {
+                g_variant_get (prop, "&s", &name);
+                manager->priv->session_is_fallback = g_strcmp0 (name, "gnome") == 0;
+                g_variant_unref (prop);
+        } else {
+                manager->priv->session_is_fallback = FALSE;
+                g_warning ("failed to get SessionName, assuming gnome\n");
+        }
+        g_object_unref (proxy);
+
         apply_input_sources_settings (manager->priv->input_sources_settings, NULL, 0, manager);
-#endif
         /* apply current settings before we install the callback */
         g_debug ("Started the keyboard plugin, applying all settings");
         apply_all_settings (manager);
