@@ -35,6 +35,7 @@
 #include "gsd-updates-refresh.h"
 #include "gsd-updates-common.h"
 #include "gnome-settings-profile.h"
+#include "gnome-settings-session.h"
 
 #define GSD_UPDATES_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_UPDATES_MANAGER, GsdUpdatesManagerPrivate))
 
@@ -694,7 +695,7 @@ auto_download_updates (GsdUpdatesManager *manager)
                 package_ids[i] = g_strdup (pk_package_get_id (pkg));
         }
 
-#if PK_CHECK_VERSION(0,8,1)
+#if PK_CHECK_VERSION(0,1,8)
         /* we've set only-download in PkTask */
         pk_task_update_packages_async (manager->priv->task,
                                        package_ids,
@@ -1258,9 +1259,6 @@ check_offline_update_cb (gpointer user_data)
                 message = ngettext ("An important OS update has been installed.",
                                     "Important OS updates have been installed.",
                                     num_packages);
-
-                /* no need to keep the file around anymore */
-                clear_offline_updates_message ();
         } else {
                 /* get error details */
                 manager->priv->offline_update_error = pk_error_new ();
@@ -1345,7 +1343,7 @@ gsd_updates_manager_start (GsdUpdatesManager *manager,
         g_object_set (manager->priv->task,
                       "background", TRUE,
                       "interactive", FALSE,
-#if PK_CHECK_VERSION(0,8,1)
+#if PK_CHECK_VERSION(0,1,8)
                       "only-download", TRUE,
 #endif
                       NULL);
@@ -1385,14 +1383,7 @@ gsd_updates_manager_start (GsdUpdatesManager *manager,
 
         /* use gnome-session for the idle detection */
         manager->priv->proxy_session =
-                g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
-                                               G_DBUS_PROXY_FLAGS_NONE,
-                                               NULL, /* GDBusInterfaceInfo */
-                                               "org.gnome.SessionManager",
-                                               "/org/gnome/SessionManager",
-                                               "org.gnome.SessionManager",
-                                               manager->priv->cancellable,
-                                               error);
+                gnome_settings_session_get_session_proxy ();
         if (manager->priv->proxy_session == NULL)
                 goto out;
 
