@@ -775,10 +775,12 @@ find_output (GnomeRRScreen  *rr_screen,
 	if (rr_output == NULL) {
 		if (gsd_wacom_device_is_screen_tablet (device)) {
 			rr_output = find_output_by_heuristic (rr_screen, device);
-			if (rr_output == NULL)
+			if (rr_output == NULL) {
 				g_warning ("No fuzzy match based on heuristics was found.");
-			else
+			} else {
 				g_warning ("Automatically mapping tablet to heuristically-found display.");
+				set_display_by_output (device, rr_output);
+			}
 		}
 	}
 
@@ -999,30 +1001,61 @@ gsd_wacom_device_add_ring_modes (WacomDevice      *wacom_device,
 {
 	GList *l;
 	guint num_modes;
+	guint group;
 	guint i;
 	char *name, *id;
 
 	l = NULL;
 
 	if ((direction & WACOM_BUTTON_POSITION_LEFT) && libwacom_has_ring (wacom_device)) {
-		/* There is a ring so there must be at least one mode */
-		num_modes = MAX (1, libwacom_get_ring_num_modes (wacom_device));
-		for (i = 1; i <= num_modes; i++) {
-			name = g_strdup_printf (_("Left Ring Mode #%d"), i);
-			id = g_strdup_printf ("left-ring-mode-%d", i);
-			l = g_list_append (l, gsd_wacom_tablet_button_new (name, id, settings_path, WACOM_TABLET_BUTTON_TYPE_ELEVATOR, flags_to_group (WACOM_BUTTON_RING_MODESWITCH), i - 1));
-			g_free (name);
-			g_free (id);
+		num_modes = libwacom_get_ring_num_modes (wacom_device);
+		group = flags_to_group (WACOM_BUTTON_RING_MODESWITCH);
+		if (num_modes == 0) {
+			/* If no mode is available, we use "left-ring-mode-1" for backward compat */
+			l = g_list_append (l, gsd_wacom_tablet_button_new (_("Left Ring"),
+									   "left-ring-mode-1",
+									   settings_path,
+									   WACOM_TABLET_BUTTON_TYPE_ELEVATOR,
+									   group,
+									   0));
+		} else {
+			for (i = 1; i <= num_modes; i++) {
+				name = g_strdup_printf (_("Left Ring Mode #%d"), i);
+				id = g_strdup_printf ("left-ring-mode-%d", i);
+				l = g_list_append (l, gsd_wacom_tablet_button_new (name,
+				                                                   id,
+				                                                   settings_path,
+				                                                   WACOM_TABLET_BUTTON_TYPE_ELEVATOR,
+				                                                   group,
+				                                                   i - 1));
+				g_free (name);
+				g_free (id);
+			}
 		}
 	} else if ((direction & WACOM_BUTTON_POSITION_RIGHT) && libwacom_has_ring2 (wacom_device)) {
-		/* There is a ring so there must be at least one mode */
-		num_modes = MAX (1, libwacom_get_ring2_num_modes (wacom_device));
-		for (i = 1; i <= num_modes; i++) {
-			name = g_strdup_printf (_("Right Ring Mode #%d"), i);
-			id = g_strdup_printf ("right-ring-mode-%d", i);
-			l = g_list_append (l, gsd_wacom_tablet_button_new (name, id, settings_path, WACOM_TABLET_BUTTON_TYPE_ELEVATOR, flags_to_group (WACOM_BUTTON_RING2_MODESWITCH), i - 1));
-			g_free (name);
-			g_free (id);
+		num_modes = libwacom_get_ring2_num_modes (wacom_device);
+		group = flags_to_group (WACOM_BUTTON_RING2_MODESWITCH);
+		if (num_modes == 0) {
+			/* If no mode is available, we use "right-ring-mode-1" for backward compat */
+			l = g_list_append (l, gsd_wacom_tablet_button_new (_("Right Ring"),
+									   "right-ring-mode-1",
+									   settings_path,
+									   WACOM_TABLET_BUTTON_TYPE_ELEVATOR,
+									   group,
+									   0));
+		} else {
+			for (i = 1; i <= num_modes; i++) {
+				name = g_strdup_printf (_("Right Ring Mode #%d"), i);
+				id = g_strdup_printf ("right-ring-mode-%d", i);
+				l = g_list_append (l, gsd_wacom_tablet_button_new (name,
+				                                                   id,
+				                                                   settings_path,
+				                                                   WACOM_TABLET_BUTTON_TYPE_ELEVATOR,
+				                                                   group,
+				                                                   i - 1));
+				g_free (name);
+				g_free (id);
+			}
 		}
 	}
 
@@ -1037,6 +1070,7 @@ gsd_wacom_device_add_strip_modes (WacomDevice      *wacom_device,
 	GList *l;
 	guint num_modes;
 	guint num_strips;
+	guint group;
 	guint i;
 	char *name, *id;
 
@@ -1046,24 +1080,54 @@ gsd_wacom_device_add_strip_modes (WacomDevice      *wacom_device,
 		g_warning ("Unhandled number of touchstrips: %d", num_strips);
 
 	if ((direction & WACOM_BUTTON_POSITION_LEFT) && num_strips >= 1) {
-		/* There is a strip so there must be at least one mode */
-		num_modes = MAX (1, libwacom_get_strips_num_modes (wacom_device));
-		for (i = 1; i <= num_modes; i++) {
-			name = g_strdup_printf (_("Left Touchstrip Mode #%d"), i);
-			id = g_strdup_printf ("left-strip-mode-%d", i);
-			l = g_list_append (l, gsd_wacom_tablet_button_new (name, id, settings_path, WACOM_TABLET_BUTTON_TYPE_ELEVATOR, flags_to_group (WACOM_BUTTON_TOUCHSTRIP_MODESWITCH), i - 1));
-			g_free (name);
-			g_free (id);
+		num_modes = libwacom_get_strips_num_modes (wacom_device);
+		group = flags_to_group (WACOM_BUTTON_TOUCHSTRIP_MODESWITCH);
+		if (num_modes == 0) {
+			/* If no mode is available, we use "left-strip-mode-1" for backward compat */
+			l = g_list_append (l, gsd_wacom_tablet_button_new (_("Left Touchstrip"),
+									   "left-strip-mode-1",
+									   settings_path,
+									   WACOM_TABLET_BUTTON_TYPE_ELEVATOR,
+									   group,
+									   0));
+		} else {
+			for (i = 1; i <= num_modes; i++) {
+				name = g_strdup_printf (_("Left Touchstrip Mode #%d"), i);
+				id = g_strdup_printf ("left-strip-mode-%d", i);
+				l = g_list_append (l, gsd_wacom_tablet_button_new (name,
+				                                                   id,
+				                                                   settings_path,
+				                                                   WACOM_TABLET_BUTTON_TYPE_ELEVATOR,
+				                                                   group,
+				                                                   i - 1));
+				g_free (name);
+				g_free (id);
+			}
 		}
 	} else if ((direction & WACOM_BUTTON_POSITION_RIGHT) && num_strips >= 2) {
-		/* There is a strip so there must be at least one mode */
-		num_modes = MAX (1, libwacom_get_strips_num_modes (wacom_device));
-		for (i = 1; i <= num_modes; i++) {
-			name = g_strdup_printf (_("Right Touchstrip Mode #%d"), i);
-			id = g_strdup_printf ("right-strip-mode-%d", i);
-			l = g_list_append (l, gsd_wacom_tablet_button_new (name, id, settings_path, WACOM_TABLET_BUTTON_TYPE_ELEVATOR, flags_to_group (WACOM_BUTTON_TOUCHSTRIP2_MODESWITCH), i - 1));
-			g_free (name);
-			g_free (id);
+		num_modes = libwacom_get_strips_num_modes (wacom_device);
+		group = flags_to_group (WACOM_BUTTON_TOUCHSTRIP2_MODESWITCH);
+		if (num_modes == 0) {
+			/* If no mode is available, we use "right-strip-mode-1" for backward compat */
+			l = g_list_append (l, gsd_wacom_tablet_button_new (_("Right Touchstrip"),
+									   "right-strip-mode-1",
+									   settings_path,
+									   WACOM_TABLET_BUTTON_TYPE_ELEVATOR,
+									   group,
+									   0));
+		} else {
+			for (i = 1; i <= num_modes; i++) {
+				name = g_strdup_printf (_("Right Touchstrip Mode #%d"), i);
+				id = g_strdup_printf ("right-strip-mode-%d", i);
+				l = g_list_append (l, gsd_wacom_tablet_button_new (name,
+				                                                   id,
+				                                                   settings_path,
+				                                                   WACOM_TABLET_BUTTON_TYPE_ELEVATOR,
+				                                                   group,
+				                                                   i - 1));
+				g_free (name);
+				g_free (id);
+			}
 		}
 	}
 
